#!/usr/bin/env rexx
/* import classes from the JFreeChart API */
   call bsf.loadClass      "org.jfree.chart.ChartUtilities"                      , "ChartUtilities"
   call bsf.loadClass      "org.jfree.chart.ChartFactory"                        , "ChartFactory"
   call bsf.importClass    "org.jfree.chart.ChartFrame"                          , "ChartFrame"
   call bsf.importClass    "org.jfree.chart.ChartPanel"                          , "ChartPanel"
   call bsf.importClass    "org.jfree.data.general.DefaultPieDataset"            , "DefaultPieDataset"
   call bsf.importClass    "org.jfree.data.time.TimeSeries"                      , "TimeSeries"
   call bsf.importClass    "org.jfree.data.time.Day"                             , "Day"
   call bsf.loadClass    "org.jfree.data.xy.XYDataset"                           , "XYDataset"
   call bsf.importClass    "org.jfree.data.time.TimeSeriesCollection"            , "TimeSeriesCollection"
   call bsf.importClass    "java.io.File"                                        , "JFile"
   call bsf.import         "javafx.scene.control.Alert"                          , "fxAlert"
   call bsf.import         "javafx.scene.control.Alert$AlertType"                , "fxAlertType"         -- nested enumeration class, hence "$" instead of "."
   call bsf.import         "javafx.scene.control.ButtonType"                     , "fxButtonType"
   call bsf.import         "javafx.scene.control.ButtonBar$ButtonData"           , "fxButtonData"   -- nested enumeration class, hence "$" instead of "."
   call bsf.import         "javafx.scene.control.ChoiceDialog"                   , "fxChoiceDialog"
   call bsf.import         "javafx.scene.control.Dialog"                         , "fxDialog"
   call bsf.import         "javafx.application.Platform"                         , "fxPlatform"

/*SAVING ENTRY BEGIN ##########################################################################################*/
::routine save public
use arg slotDir
   /*Connect to Database*/
      call sqlite_conn
      tmpColl = result
      conn = tmpColl
      statement = conn~createStatement

   /*Connect to the ID Fields from FXML File*/
   /* @get(sportart_cb) */
   /* @get(date)        */
   /* @get(km_field)    */
   /* @get(failure_save)    */
   
      if (km_field~getText = .nil | sportart_cb~getValue = .nil | date~getValue = .nil) then do
         failure_save~setText("Not all required fields have been filled in.")
         say "provided not enough information to save"
      end

      else do
         container = .array~of(3)

         container[1] = km_field~getText
         container[2] = sportart_cb~getValue
         container[3] = date~getValue~toString

         input = .array~of(3)
         input1 = .bsf~new("java.lang.String",container[1])
         input2 = .bsf~new("java.lang.String",container[2])
         input3 = .bsf~new("java.lang.String",container[3]) 

         query_insert = statement~execute("Insert INTO sporttable (km, sport, date) VALUES('"input1~toString"','"input2~toString"', '"input3~toString"');")



         query = statement~executeQuery('SELECT * from sporttable')
         do while query~next
            say query~getString("sport") "       |" query~getString("km")"km am:" query~getString("date")
         end
      end
   
/*SAVING ENTRY END ##########################################################################################*/

/*STATISTIC BEGIN ##########################################################################################*/
::routine stat public
use arg slotDir
   /*Connect to Database*/
   call sqlite_conn
   tmpColl = result
   conn = tmpColl
   statement = conn~createStatement

   /*Connect to the ID Fields from FXML File*/
   /* @get(sportart_cb) */
   /* @get(failure_save)    */
   sport = sportart_cb~getValue
   allSports = sportart_cb~getItems~toArray
   failure_save~setText(.nil)
 

   res=calcTimeSeriesCollection(conn, sport, allSports)
 
   /*Invoke Class Chartdialog with sport and res(Finished TimeSeriesCollection)*/
   .chartDialog~chart(sport, res)
   

::routine calcTimeSeriesCollection
   use arg conn, sport, allSports

   statement = conn~createStatement

   /* Create a TimeSeriesCollection*/
   TimeSeriesCollection = .TimeSeriesCollection~new

   if sport = .nil then do
      do i over allSports
         TimeSeriesCollection~addSeries(calcSeries(conn, i))
      end
   end
   else do
      TimeSeriesCollection~addSeries(calcSeries(conn, sport))
   end
      
   return TimeSeriesCollection


::routine calcSeries public
   use arg conn, sport

   statement = conn~createStatement
    

      series = .TimeSeries~new(sport)
      input = .bsf~new("java.lang.String",sport)
      query = statement~executeQuery("SELECT * FROM sporttable WHERE sport = '"sport"' ")
      do while query~next
         km  = query~getString("km")
         date = query~getString("date")
         year = date[1,4]
         month =date[6,2]
         day = date[9,2]
         series~add(.Day~new(day,month,year), km)
      end
      return series
   




/*STATISTIC END ##########################################################################################*/

/*OUPUT BEGIN ##########################################################################################*/
::routine output public
   use arg slotDir
      /*Connect to Database*/
      call sqlite_conn
      tmpColl = result
      conn = tmpColl
      statement = conn~createStatement

      /*Connect to the ID Fields from FXML File*/
      /* @get(op1)        */
      /* @get(op2)        */
      /* @get(op3)        */
      /* @get(op4)        */
      /* @get(op5)        */
      /* @get(op6)        */
      /* @get(op7)        */
      /* @get(op8)        */
      /* @get(op9)        */
      /* @get(op10)        */
      /* @get(failure_save)    */

      failure_save~setText(.nil)
      op10~setText(.nil)
      op9~setText(.nil)
      op8~setText(.nil)
      op7~setText(.nil)
      op6~setText(.nil)
      op5~setText(.nil)
      op4~setText(.nil)
      op3~setText(.nil)
      op2~setText(.nil)
      op1~setText(.nil)

      outputArray = .array~new
      query= statement~executeQuery('SELECT * FROM sporttable ORDER BY date(date)')
      do while query~next
         itemArray = .array~new(3)
         itemArray~append(query~getString("sport"))
         itemArray~append(query~getString("km"))
         itemArray~append(query~getString("date"))
         outputArray~append(itemArray)
      end
      do i over outputArray
         op10~setText(op9~getText)
         op9~setText(op8~getText)
         op8~setText(op7~getText)
         op7~setText(op6~getText)
         op6~setText(op5~getText)
         op5~setText(op4~getText)
         op4~setText(op3~getText)
         op3~setText(op2~getText)
         op2~setText(op1~getText)
         op1~setText(i[3]"   |" i[2]"km   " i[1])
      end
/*OUPUT END ##########################################################################################*/

/*CLEARING BEGIN ##########################################################################################*/
::routine clearAll public
   use arg slotDir
      /*Connect to the ID Fields from FXML File*/
      /* @get(sportart_cb) */
      /* @get(date)        */
      /* @get(km_field)    */
      /* @get(op1)        */
      /* @get(op2)        */
      /* @get(op3)        */
      /* @get(op4)        */
      /* @get(op5)        */
      /* @get(op6)        */
      /* @get(op7)        */
      /* @get(op8)        */
      /* @get(op9)        */
      /* @get(op10)        */
      /* @get(failure_save)        */

      date~setValue(.nil)
      km_field~setText(.nil)
      sportart_cb~setValue(.nil)
      failure_save~setText(.nil)
      
      op10~setText(.nil)
      op9~setText(.nil)
      op8~setText(.nil)
      op7~setText(.nil)
      op6~setText(.nil)
      op5~setText(.nil)
      op4~setText(.nil)
      op3~setText(.nil)
      op2~setText(.nil)
      op1~setText(.nil)

      say "Everything cleared"
/*CLEARING END ##########################################################################################*/

/*CHART BEGIN ##########################################################################################*/
::class chartDialog

::method chart class
   use arg sport, timeSeriesCollection
   filename="chart.png"
   
   if sport = .nil then sport = "all Sports"

   /* generates the chart.. for explanation of the input, look at the JFreeChart API : http://www.jfree.org/jfreechart/javadoc/  */
   chart = .ChartFactory~createTimeSeriesChart("Your statistic for" sport,"Date","Kilometer", TimeSeriesCollection)

   chartFile=.JFile~new(fileName)
   .ChartUtilities~saveChartAsPNG(chartFile, chart, 800,500)  -- save the file as PNG

   dialog=.fxDialog~new   -- create a Dialog and customize it: has OK and CANCEL button
   dialogPane=dialog~dialogPane

   image=.bsf~new("javafx.scene.image.Image", "file:chart.png")
   dialogPane~scene~window~icons~add(image)  -- make it a system menu icon (upper left-hand corner)

      -- this time we resize the ImageView to 64 pixel
   imageView=.fxImageView~new(image)~~setFitWidth(1400)~~setPreserveRatio(.true)~~setSmooth(.true)

   dialog~graphic=imageView
   dialog~title      ="Sport Update Statistics"

   ButtonType=.fxButtonType~new("OK", .fxButtonData~OK_DONE) -- use as the default push button
   jarrBT=bsf.createJavaArrayOf(.fxButtonType, ButtonType)
   dialogPane~buttonTypes~setAll(jarrBT)    -- dialog.getButtonTypes().setAll(...)
      -- define grid and the fields and labels it should contain
   grid=.fxGridPane~new ~~setHGap(10) ~~setVGap(10) ~~setPadding(.bsf~new("javafx.geometry.Insets", 20,150,10,10))

   rexxDir=.directory~new
   rexxDir~ButtonType=ButtonType
   jobj=bsfCreateRexxProxy(.demoCustomizedDialogListener~new,rexxDir,"javafx.beans.value.ChangeListener","javafx.util.Callback")
   dialog~setResultConverter(jobj)   

   dialogPane~content=grid

   res=dialog~showAndWait      -- show the dialog, wait until user closes it, returns a java.util.Optional (isPresent(), get(), ...)

   if res~isPresent then return 0
   else return 1

/*CHART END ##########################################################################################*/

::method noSport class

   dialog=.fxDialog~new   -- create a Dialog and customize it: has OK and CANCEL button
   dialogPane=dialog~dialogPane

  
   dialog~title      ="Attention!"
   dialog~headerText ="You chose no Sport. Please choose."

   ButtonType=.fxButtonType~new("OK", .fxButtonData~OK_DONE) -- use as the default push button
   jarrBT=bsf.createJavaArrayOf(.fxButtonType, ButtonType)
   dialogPane~buttonTypes~setAll(jarrBT)    -- dialog.getButtonTypes().setAll(...)
      -- define grid and the fields and labels it should contain
   grid=.fxGridPane~new ~~setHGap(10) ~~setVGap(10) ~~setPadding(.bsf~new("javafx.geometry.Insets", 20,150,10,10))

   rexxDir=.directory~new
   rexxDir~ButtonType=ButtonType
   jobj=bsfCreateRexxProxy(.demoCustomizedDialogListener~new,rexxDir,"javafx.beans.value.ChangeListener","javafx.util.Callback")
   dialog~setResultConverter(jobj)   

   dialogPane~content=grid

   res=dialog~showAndWait      -- show the dialog, wait until user closes it, returns a java.util.Optional (isPresent(), get(), ...)

   if res~isPresent then return 0
   else return 1

::class demoCustomizedDialogListener

-- javafx.beans.value.ChangeListener<T>: void changed(ObservableValue<? extends T> observable, T oldValue, T newValue)
::method changed              -- whenever the userName TextField gets edited
  use arg observable, oldValue, newValue, slotDir

-- javafx.util.Callback<P,R>: R call(P param): used by the Dialog's "ResultConverter"
::method call
   use arg param, slotDir
  --say "\2\" self"::call:    param="pp(param) "slotDir="pp(slotDir)
  --rexxDir=slotDir~userData    -- fetch the Rexx directory we supplied from the slotDir directory (supplied by BSF4ooRexx)

/*Seems just working fine without that. Dont know*/
  --if param~text==rexxDir~ButtonType~text then
     --return .bsf~new("javafx.util.Pair")
  return .nil


  ::requires database.rex
  ::requires BSF.CLS